/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.1 Stauverlaufsanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.anastau.bestimmungstau.indikator;

import de.bsvrz.ibv.anastau.migration.modell.verkehr.objekte.StoerfallIndikator;

/**
 * Interface für den Zugriff auf die Daten eines Störfallindikators.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public interface ISobIndikator {
	/**
	 * liefert den verwendeten Störfallindikator aus der
	 * Datenverteilerkonfiguration.
	 *
	 * @return den Indikator
	 */
	StoerfallIndikator getIndikator();

	/**
	 * liefert die zugeordnete Länge des Indikators auf der Straße.
	 *
	 * @return die Länge in Metern
	 */
	double getLaenge();

	/**
	 * liefert den Namen des Indikators.
	 *
	 * @return den Namen
	 */
	String getName();

	/**
	 * liefert den in Fahrtrichtung liegenden nächsten Indikator.
	 *
	 * @return den Indikator
	 */
	ISobIndikator getNext();

	/**
	 * liefert die Position des Indikators relativ zum Startpunkt der Straße (in
	 * Metern).
	 *
	 * @return die Position
	 */
	double getPosition();

	/**
	 * liefert den entgegengesetzt zur Fahrtrichtung liegenden nächsten
	 * Indikator.
	 *
	 * @return den Indikator
	 */
	ISobIndikator getPrev();

	/**
	 * liefert die Richtung des Verkehrsflusses den der Indikator auswertet.
	 *
	 * @return die Richtung
	 */
	IFahrtRichtung getRichtung();

	/**
	 * liefert die aktualle Situation die vom Indikator ermittelt wurde.
	 *
	 * @return die Situation
	 */
	SobStoerfallSituation getSituation();

	/**
	 * liefert die Position in der ein durch den Indikator bewirkter Staubereich
	 * beginnt (in Metern).
	 *
	 * @return die Position
	 */
	double getStauBeginn();

	/**
	 * liefert die Position in der ein durch den Indikator bewirkter Staubereich
	 * endet (in Metern).
	 *
	 * @return die Position
	 */
	double getStauEnde();

	/**
	 * liefert die Straße, der der Indikator zugeordnet ist.
	 *
	 * @return die Straße
	 */
	ISobStrasse getStrasse();

	/**
	 * liefert eine Aussage, ob freier Verkehr gemeldet wurde.
	 *
	 * @return true, wenn der Indikator freien Verkehr meldet
	 */
	boolean isFreierVerkehr();

	/**
	 * ermittelt, ob der Status des Indikators verändert wurde.
	 *
	 * @return den Status
	 */
	boolean isGeaendert();

	/**
	 * setzt den Änderungsstatus des Indikators.
	 *
	 * @param status
	 *            der Zielstatus
	 */
	void setGeaendert(boolean status);

	/**
	 * setzt die Position des Indikators.
	 *
	 * @param position
	 *            die Position
	 */
	void setPosition(double position);

	/**
	 * sichert die aktuelle Situation des Indikators.
	 */
	void sichereSituation();
}
